/**
 * Render sass source into file.
 * @memberof apeman-commons-compiling
 * @function renderSass
 * @param {string} src - Source filename or source string.
 * @param {string} dest - Destination file path.
 * @param {object} [options] - Optional settings.
 * @param {string[]} [options.includePaths] - Paths to include.
 * @param {boolean} [options.log] - Should emit log.
 * @param {function} [callback] - Callback when done.
 */

"use strict";

var async = require('async'),
    fs = require('fs'),
    writeout = require('writeout'),
    argx = require('argx'),
    colorprint = require('colorprint'),
    sass = require('node-sass');

/** @lends renderSass */
function renderSass(src, dest, options, callback) {
    var args = argx(arguments);
    callback = args.pop('function') || argx.noop;
    options = args.pop('object') || {};
    src = args.shift();
    dest = args.pop('string');

    if (!dest) {
        callback(new Error('dest is required.'));
        return;
    }

    async.waterfall([
        function (callback) {
            fs.exists(src, function (exists) {
                if (exists) {
                    fs.readFile(src, callback);
                } else {
                    callback(null, src);
                }
            });
        },
        function (src, callback) {
            sass.render({
                data: src,
                outFile: dest,
                includePaths: [].concat(options.includePaths || [])
            }, callback);
        },
        function (reslut, callback) {
            writeout(dest, reslut.css, {
                mkdirp: true,
                skipIfIdentical: true
            }, callback);
        },
        function (result, callback) {
            var shouldLog = !result.skipped && options.log;
            if (shouldLog) {
                colorprint.debug('File generated:', dest);
            }
            callback(null, result);
        }
    ], callback);
}

module.exports = renderSass;
